#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import division

import os
import csv
import threading
import datetime

nr_loop = 0
d_username_filesize = dict()

def get_user_name_info(gid):
    d_user_name_info = dict()
    with open('/etc/passwd', 'r') as f:
        for line in f.readlines():
            l = line.strip().split(':')

            username = l[0]
            uid = int(l[2])
            tmp_gid = int(l[3])
            user_info = l[4]
            home_dir = l[5]

            if tmp_gid == gid:
                d_user_name_info[username] = user_info

    return d_user_name_info


def record_file_changes(out_file, d_user_name_info,  detect_formatted_filenames, start_dt, end_dt):
    global nr_loop
    global d_username_filesize

    dt_now = datetime.datetime.now()
    if dt_now > end_dt:
        return

    # record files every second
    #class threading.Timer(interval, function, args=None, kwargs=None)
    threading.Timer(1.0, record_file_changes, args=[out_file, d_user_name_info,  detect_formatted_filenames, start_dt, end_dt]).start()

    with open(out_file, 'a+') as f:
        writer = csv.writer(f)
        for username in d_user_name_info:

            for detect_formatted_filename in detect_formatted_filenames:
                detect_filename = detect_formatted_filename.format(username=username)

                filename = os.path.basename(detect_filename)
                try:
                    file_size = os.path.getsize(detect_filename)
                except Exception as e:
                    print(username, str(e))
                    continue

                dt = datetime.datetime.now()
                str_dt = dt.strftime('%Y-%m-%d %H:%M:%S')

                student_id = d_user_name_info[username].split('-')[1]
                l = [str_dt, student_id, filename, file_size]

                # writer to file if file size changed
                if d_username_filesize[username] != file_size or nr_loop==0:
                    writer.writerow(l)
                    d_username_filesize[username] = file_size

        nr_loop += 1


def main():
    global d_username_filesize
    # Username:Password:User ID:Group ID:User Info:Home Directory:Default Shell
    #linxintao:x:19132:191112:201921135048:/home/linxintao:/bin/bash

    # Step 1: read file
    gid = 201112
    d_user_name_info = get_user_name_info(gid)

    d_username_filesize = dict.fromkeys(d_user_name_info, 0)

    # Step 2: read data, and get stats
    start_dt = datetime.datetime.strptime('2022-05-30 13:55', '%Y-%m-%d %H:%M')
    end_dt = datetime.datetime.strptime('2022-05-30 15:40', '%Y-%m-%d %H:%M')

    '''
    start_dt = datetime.datetime.strptime('2021-04-12 02:10', '%Y-%m-%d %H:%M')
    end_dt = datetime.datetime.strptime('2021-04-12 02:11', '%Y-%m-%d %H:%M')
    '''

    out_file = 'file_changes_exp4_reader_writer_pv_2022.csv'
    if os.path.isfile(out_file):
        os.remove(out_file)

    #detect_formatted_filename = '/home/{username}/os_exp/exp2_process/fork_two_children.c'
    #detect_formatted_filename = '/home/{username}/os_exp/exp3_pipe/pipe.c'
    detect_formatted_filenames = ['/home/{username}/os_exp/exp4_sync_pv/process_synchronization_pv.c', '/home/{username}/os_exp/exp4_sync_pv/semaphore.h']
    detect_formatted_filenames = ['/home/{username}/os_exp/exp5_reader_writer_pv/reader_writer_pv.c']

    # Start a Function at a Given Time
    dt_now = datetime.datetime.now()
    delay = (start_dt - dt_now).total_seconds()
    if delay > 0:
        print('delay', delay)
        threading.Timer(delay, record_file_changes, args=[out_file, d_user_name_info,  detect_formatted_filename, start_dt, end_dt]).start()
    else:
        record_file_changes(out_file, d_user_name_info,  detect_formatted_filenames, start_dt, end_dt)

if __name__ == '__main__':
    main()
